# What are you looking for?
# Leaking assets comes with heavy punishments!

from PIL import Image
from PIL import ImageEnhance
import numpy as np
import os
import json
import math

models = ''
spreadsheet = ''
model_data = 7922000
texture_referencer = ''
data_path = '../../../../saves/TERF_infinity_normalmap/datapacks/TERF/data/'

def generate_image(image, name, color, saturation):
    """
    Apply color to a black and white image with a specified saturation.
    
    Parameters:
        image: PIL.Image
            Input black and white image.
        color: tuple (r, g, b)
            RGB color tuple.
        saturation: float
            Saturation level, should be between 0 and 1.
    
    Returns:
        PIL.Image
            Colorized image.
    """
    # Convert image to RGB if it's not already in RGB mode
    image_rgb = image.convert('RGBA')
    
    # Create numpy array from the image
    np_image = np.array(image_rgb)
    
    # Split the image into separate color channels
    r, g, b = np_image[:, :, 0], np_image[:, :, 1], np_image[:, :, 2]
    
    # Adjust saturation
    gray = 0.2989 * r + 0.5870 * g + 0.1140 * b
    adjusted_r = gray + saturation * (r - gray)
    adjusted_g = gray + saturation * (g - gray)
    adjusted_b = gray + saturation * (b - gray)
    
    # Apply color
    np_image[:, :, 0] = adjusted_r * color[0]
    np_image[:, :, 1] = adjusted_g * color[1]
    np_image[:, :, 2] = adjusted_b * color[2]
    
    # Convert numpy array back to image
    colorized_image = Image.fromarray(np.uint8(np_image))
    
    colorized_image.save('textures/item/materials/' + name + '.png')

def capitalize_words(input_string):
    # Split the string into words
    words = input_string.split()
    
    # Capitalize the first letter of each word
    capitalized_words = [word.capitalize() for word in words]
    
    # Join the capitalized words back into a single string
    capitalized_string = ' '.join(capitalized_words)
    
    return capitalized_string

def generate_item(texture, type, obj):
        global models
        global texture_referencer
        global model_data
        global spreadsheet

        name = obj['name'] + type

        generate_image(texture, name, (obj['r'],obj['g'],obj['b']), obj['sat'])

        # Generate models
        models = models + ',\n  { "predicate": {"custom_model_data": ' + str(model_data) +'}, "model": "item/materials/' + name + '"}'

        # Generate texture referencers
        with open('models/item/materials/' + name + '.json', 'w') as file:
            file.write(texture_referencer.replace('$NAME', name))

        # Generate spreadsheet
        spreadsheet = spreadsheet + '{id:"' + name + '",cmd:' + str(model_data) + ',name:"' + capitalize_words(name.replace('_',' ')) + '"},'

def delete_all_files_in_folder(folder_path):
    # List all files in the folder
    for filename in os.listdir(folder_path):
        file_path = os.path.join(folder_path, filename)
        try:
            # Check if it is a file and delete it
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.remove(file_path)
        except Exception as e:
            print(f'Failed to delete {file_path}. Reason: {e}')

###################################################################
#remove old files
delete_all_files_in_folder('models/item/materials')
delete_all_files_in_folder('textures/item/materials')
delete_all_files_in_folder(data_path + 'terf/recipes/generated')

# Load resource images
dust = Image.open('generate/dust.png')
ingot = Image.open('generate/ingot.png')
nugget = Image.open('generate/nugget.png')
plate = Image.open('generate/plate.png')
wire = Image.open('generate/wire.png')
gem = Image.open('generate/gem.png')

# Set the materials
materials = []
with open('generate/materials.txt', 'r') as file:
    for line in file:
        if not (line.startswith('\n') or line.startswith('#')):
            obj = json.loads(line)
            materials.append(obj)

with open('generate/model_referencer.txt', 'r') as file:
    for line in file:
        model_referencer = file.read()

with open('generate/texture_referencer.txt', 'r') as file:
    for line in file:
        texture_referencer = file.read()

# Generate images, texture referencers and add to models

for obj in materials:
    generate_item(dust, "_dust", obj)
    model_data += 1
    if obj['ingot'] == 1:
        generate_item(ingot, "_ingot", obj)
    model_data += 1
    if obj['ingot'] == 1:
        generate_item(nugget, "_nugget", obj)
    model_data += 1
    if obj['gem'] == 1:
        generate_item(gem, "", obj)
    model_data += 1
    if obj['plate'] == 1:
        generate_item(plate, "_plate", obj)
    model_data += 1
    if obj['wire'] == 1:
        generate_item(wire, "_wire", obj)

    model_data = math.ceil(model_data/10) * 10

# Generate model
with open('models/item/recovery_compass.json', 'w') as file:
    file.write(model_referencer.replace('$MODELS', models))

# Generate recipes
with open(data_path + 'terf/recipes/generated/sk', 'w') as file:
    file.write(model_referencer.replace('$MODELS', models))

# Generate data spreadsheet
with open(data_path + 'terf/functions/generated.mcfunction', 'w') as file:
    file.write("#This function is modified by the python script in the resource pack!\ndata modify storage terf:constants generated_materials set value [" + spreadsheet + "]")
    